# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: GroupsService.py,v 1.8 2003/02/17 18:21:35 zagy Exp $
from AccessControl import ClassSecurityInfo
import Globals
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

import GroupsPermissions
from GroupsErrors import GroupsError
    
class GroupsService:
    """Attach users to groups. This is the most simple implementation.
    Other implementations may get groups data from LDAP, etc.

    There are two kinds of groups:

        * normal group (ZODB defined, LDAP, or both etc).
        * virtual group (a user is in a virtual group if that user is in a
                         defined set of normal groups)
    """
    security = ClassSecurityInfo()
    
    def __init__(self):
        self._users = {}
        self._virtual_groups = {}
        self._group_to_virtual = {}
        self._normal_groups = {}
        self._user_to_managed_normal_groups = {}
        self._user_to_managed_virtual_groups = {}

    # MANIPULATORS
    def addGroupToManagedNormalGroups(self, user, group):
        self._user_to_managed_normal_groups.setdefault(user, {})[group] = 1
        self._p_changed = 1

    def removeGroupFromManagedNormalGroups(self, user, group):
        del self._user_to_managed_normal_groups[user][group]
        self._p_changed = 1

    def addGroupToManagedVirtualGroups(self, user, group):
        self._user_to_managed_virtual_groups.setdefault(user, {})[group] = 1
        self._p_changed = 1

    def removeGroupFromManagedVirtualGroups(self, user, group):
        del self._user_to_managed_virtual_groups[user][group]
        self._p_changed = 1

    def getManagedNormalGroups(self, user):
        return self._user_to_managed_normal_groups.get(user, {})

    def getManagedVirtualGroups(self, user):
        return self._user_to_managed_virtual_groups.get(user, {})

    security.declareProtected(GroupsPermissions.ChangeGroups,
                              'addUserToZODBGroup')
    def addUserToZODBGroup(self, userid, group):
        """Add a user to a group defined in the ZODB.
        """
        self._users.setdefault(userid, {})[group] = 1
        self._p_changed = 1

    security.declareProtected(GroupsPermissions.ChangeGroups,
                              'removeUserFromZODBGroup')
    def removeUserFromZODBGroup(self, userid, group):
        """Remove a user from a group defined in the ZODB.
        """
        del self._users[userid][group]
        self._p_changed = 1

    security.declareProtected(GroupsPermissions.ChangeGroups,
                              'addNormalGroup')
    def addNormalGroup(self, group):
        """Register the name of a normal group. This is intended to be used by
        the user interface mainly; in the logic unregistered groups will work
        as well. Any externally defined group (say by LDAP) is also
        considered to be a normal group.
        """
        self._normal_groups[group] = 1
        self._p_changed = 1

    security.declareProtected(GroupsPermissions.ChangeGroups,
                              'removeNormalGroup')
    def removeNormalGroup(self, group):
        """Unregister a group.
        """
        if self._group_to_virtual.has_key(group):
            raise GroupsError, "This group is used by a virtual group"
        del self._normal_groups[group]
        del self._group_to_virtual[group]
        self._p_changed = 1
        
    security.declareProtected(
        GroupsPermissions.ChangeGroups, 'addVirtualGroup')
    def addVirtualGroup(self, virtual_group):
        """Add a new virtual group 
        """
        if self._normal_groups.has_key(virtual_group):
            raise GroupsError, "Can't create virtual group that is a normal group."
        self._virtual_groups[virtual_group] = []
        self._p_changed = 1

    security.declareProtected(
        GroupsPermissions.ChangeGroups, 'removeVirtualGroup')
    def removeVirtualGroup(self, virtual_group):
        """Remove a virtual group.
        """
        del self._virtual_groups[virtual_group]
        for group, virtual_groups  in self._group_to_virtual.items():
            if virtual_group in virtual_groups:
                virtual_groups.remove(virtual_group)
                self._group_to_virtual[group] = virtual_groups
        self._p_changed = 1

    security.declareProtected(
        GroupsPermissions.ChangeGroups, 'addGroupToVirtualGroup')    
    def addGroupToVirtualGroup(self, group, virtual_group):
        """Add a group to a virtual group the user
        will automatically be in.
        """
        virtual_groups = self._virtual_groups
        g_to_v = self._group_to_virtual

        if virtual_groups.has_key(group):
            raise GroupsError, "Can't create virtual group that depends on another virtual group: %s" % virtual_group
        virtual_groups[virtual_group].append(group)
        g_to_v.setdefault(group, []).append(virtual_group)
        self._p_changed = 1

    security.declareProtected(
        GroupsPermissions.ChangeGroups, 'removeGroupFromVirtualGroup')
    def removeGroupFromVirtualGroup(self, group, virtual_group):
        virtual_groups = self._virtual_groups
        g_to_v = self._group_to_virtual
        if virtual_groups.has_key(virtual_group):                        
            virtual_groups[virtual_group].remove(group)
            g_to_v[group].remove(virtual_group)
            if not g_to_v[group]:
                del g_to_v[group]
            self._p_changed = 1

    security.declareProtected(GroupsPermissions.ChangeGroups,
                              'clear')
    def clear(self):
        """Clear all group info
        """
        self._users = {}
        self._virtual_groups = {}
        self._group_to_virtual = {}
        self._normal_groups = {}
    
    def _getNormalGroups(self, user):
        """Get the normal (non-virtual) groups a user is in. Can
        be overridden by subclasses. In this case, the user only
        is in ZODB defined groups but other GroupsService implementations
        could list groups defined in LDAP, etc.
        """
        return self._users.get(user.getUserName(), {})
        
    # ACCESSORS
    security.declareProtected(
        GroupsPermissions.AccessGroups, 'getGroups')
    def getGroups(self, user):
        """Get the groups (normal and virtual) the user is in. Expects user
        object that has at least getUserName() defined.
        """
        # get the non-virtual groups the user is in
        groups = {}
        groups.update(self._getNormalGroups(user))
        # check whether we also have some virtual groups
        virtual_groups = self._virtual_groups
        g_to_v = self._group_to_virtual
        found_possible_virtual_groups = {}
        for group in groups.keys():
            for possible_virtual_group in g_to_v.get(group, []):
                count = found_possible_virtual_groups.get(
                    possible_virtual_group, 0)
                found_possible_virtual_groups[possible_virtual_group] = \
                    count + 1
        for possible_virtual_group, count in \
                found_possible_virtual_groups.items():
            if len(virtual_groups[possible_virtual_group]) == count:
                groups[possible_virtual_group] = 1        
        return groups

    security.declareProtected(GroupsPermissions.AccessGroups,
                              'isNormalGroup')
    def isNormalGroup(self, group):
        """Return true if group is a (non-virtual) normal group.
        """
        return self._normal_groups.has_key(group)

    security.declareProtected(
        GroupsPermissions.Permissions.access_contents_information, 
        'isVirtualGroup')
    def isVirtualGroup(self, group):
        """Returns true if group is a virtual group.
        """
        return self._virtual_groups.has_key(group)

    security.declareProtected(
        GroupsPermissions.Permissions.access_contents_information, 
        'isGroup')
    def isGroup(self, group):
        """Returns true if group is a group (virtual or not).
        """
        return self.isNormalGroup(group) or self.isVirtualGroup(group)

    security.declareProtected(
        GroupsPermissions.Permissions.access_contents_information,
        'listNormalGroups')
    def listNormalGroups(self):
        result = self._normal_groups.keys()
        result.sort()
        return result

    security.declareProtected(
        GroupsPermissions.Permissions.access_contents_information,
        'listVirtualGroups')
    def listVirtualGroups(self):
        result = self._virtual_groups.keys()
        result.sort()
        return result

    security.declareProtected(
        GroupsPermissions.Permissions.access_contents_information, 
        'listAllGroups')
    def listAllGroups(self):
        result = self.listNormalGroups() + self.listVirtualGroups()
        result.sort()
        return result

    security.declareProtected(
        GroupsPermissions.AccessGroups, 'listUsersInZODBGroup')
    def listUsersInZODBGroup(self, group):
        result = {}
        for user, groups in self._users.items():
            if groups.has_key(group):
                result[user] = 1
        if not result:
            return []
        return result.keys()
    
    security.declareProtected(
            GroupsPermissions.AccessGroups, 'listGroupsInVirtualGroup')
    def listGroupsInVirtualGroup(self, virtual_group):
        result = {}
        for group, virtual_groups in self._group_to_virtual.items():
            if virtual_group in virtual_groups:
                result[group] = 1
        if not result:
            return []
        return result.keys()

    security.declareProtected(
        GroupsPermissions.AccessGroups, 'groupExists')
    def groupExists(self, group):
        return self._virtual_groups.has_key(group) or self._normal_groups.has_key(group)
    
Globals.InitializeClass(GroupsService)
